package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.PerformanceBenchmarkInfo;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IPerformanceBenchmarkInfoService;
import com.ims.eval.util.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * 业绩对标管理
 *
 * @author hlf
 * @date 2024/3/21 15:58
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-performance-benchmarking")
public class PerformanceBenchmarkingManageController {

	@Resource
	private IPerformanceBenchmarkInfoService performanceBenchmarkInfoService;

	/**
	 * 业绩对标管理列表信息（分页）
	 *
	 * @param pageNum        当前记录起始索引
	 * @param pageSize       每页显示记录数
	 * @param toolCategory   工具类别（寻标、立标、标杆、达标、超标）
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 * @param indexName      指标名称
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "toolCategory") String toolCategory,
		@RequestParam(value = "manageCategory") String manageCategory,
		@RequestParam(value = "checkCycle") String checkCycle,
		@RequestParam(value = "degreeYear") Integer degreeYear,
		@RequestParam(value = "quarterlyMonth", required = false) Integer quarterlyMonth,
		@RequestParam(value = "indexName", required = false) String indexName) {
		IPage<PerformanceBenchmarkInfo> list = performanceBenchmarkInfoService.listPage(pageNum, pageSize, toolCategory, manageCategory, checkCycle, degreeYear, quarterlyMonth, indexName);
		return R.ok().data(list);
	}

	/**
	 * 修改业绩对标管理信息
	 *
	 * @param performanceBenchmarkInfo 业绩对标管理实体
	 * @return 结果
	 */
	@PostMapping(value = "/update")
	public R update(@RequestBody PerformanceBenchmarkInfo performanceBenchmarkInfo) {
		try {
			boolean b = performanceBenchmarkInfoService.updateById(performanceBenchmarkInfo);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("修改失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 导出
	 */
	@GetMapping(value = "/exportExcel", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
	public void exportExcel(@RequestParam(value = "toolCategory") String toolCategory,
							@RequestParam(value = "manageCategory") String manageCategory,
							@RequestParam(value = "checkCycle") String checkCycle,
							@RequestParam(value = "degreeYear") Integer degreeYear,
							@RequestParam(value = "quarterlyMonth", required = false) Integer quarterlyMonth,
							HttpServletResponse response) throws IOException {
		performanceBenchmarkInfoService.exportExcel(toolCategory, manageCategory, checkCycle, degreeYear, quarterlyMonth, response);
	}

	/**
	 * 导入
	 */
	@PostMapping(value = "/importExcel")
	public R importExcel(@RequestParam("file") MultipartFile file) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					PerformanceBenchmarkInfo performanceBenchmarkInfo = new PerformanceBenchmarkInfo();
					performanceBenchmarkInfo.setId(str[0]);
					performanceBenchmarkInfo.setScalingValue(Double.valueOf(str[8]));
					performanceBenchmarkInfoList.add(performanceBenchmarkInfo);
				}
				boolean b = performanceBenchmarkInfoService.updateBatchById(performanceBenchmarkInfoList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error("导入失败！");
				}
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.error("上传文件为空！");
	}
}
