package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.OrganizationRatingMapper;
import com.ims.eval.entity.*;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 年度评级 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-06-20
 */
@Service
public class OrganizationRatingServiceImpl extends ServiceImpl<OrganizationRatingMapper, OrganizationRating> implements IOrganizationRatingService {

	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Resource
	private IOrganizationEvaluationService iOrganizationEvaluationService;

	@Resource
	private IOrganizationRatingService iOrganizationRatingService;

	@Resource
	private IOrganizationEvaluationRuleService iOrganizationEvaluationRuleService;

	@Resource
	private IBinSectionService iBinSectionService;

	@Resource
	private IOrganizationRatingRuleService iOrganizationRatingRuleService;


	@Override
	public IPage<OrganizationRating> grade(List<OrganizationRating> orgRating) {
		XSSFWorkbook xwb = new XSSFWorkbook();
		Sheet st = xwb.createSheet();
		List<OrganizationRatingRule> rules = iOrganizationRatingRuleService.list(null);
		List<OrganizationRatingRule> rulesYear = rules.stream().filter(rul -> orgRating.get(0).getYear().equals(rul.getYear())).collect(Collectors.toList());

		Map<String, List<OrganizationRating>> collect = orgRating.stream().collect(Collectors.groupingBy(OrganizationRating::getBinSection));
		Map<String, List<OrganizationRating>> col = new HashMap<>();
		collect.forEach((key, value) -> CacheContext.bsnList.stream()
                .filter(b -> key.equals(b.getSectionCode()))
                .findFirst()
                .ifPresent(b -> col.put(b.getId(), value)));

		calcSCJYrating(rulesYear, col,st,xwb);
		calcQQXMrating(rulesYear, col,st,xwb);
		calcJJXMrating(rulesYear, col,st,xwb);
		calcNDrating(rulesYear, orgRating,st,xwb);
		return null;
	}


	/**
	 *年度评级
	 */
	private void calcNDrating(List<OrganizationRatingRule> list, List<OrganizationRating> orgRating,Sheet st,XSSFWorkbook xwb) {
		List<String> ls = new ArrayList<>();
		for (OrganizationRating or : orgRating) {
			ls.add(or.getId());
		}
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "年度评级".equals(orr.getCheckCycle())).collect(Collectors.toList());
		List<OrganizationRating> org = iOrganizationRatingService.listByIds(ls);
		Map<String, List<OrganizationRating>> col = org.stream().collect(Collectors.groupingBy(OrganizationRating::getBinSection));

		Map<String, List<OrganizationRating>> col2 = new HashMap<>();
		col.forEach((key, value) -> CacheContext.bsnList.stream()
			.filter(b -> key.equals(b.getSectionCode()))
			.findFirst()
			.ifPresent(b -> col2.put(b.getId(), value)));

		for (Map.Entry<String, List<OrganizationRating>> orgMap : col2.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			if (collect.isEmpty()) {
				continue;
			}
			String formula=collect.get(0).getExpression();

		for (OrganizationRating or : orgMap.getValue()) {
			if (or.getScjyRate() == null ||or.getScjyRate().isEmpty()|| or.getQqRate() == null||or.getQqRate().isEmpty()) {
				continue;
			}
			String replace = formula;
			Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");//编译成模式
			Matcher matcher = pattern.matcher(formula);
			Map<String, String> placeholders = new HashMap<>();
			String key=null;
			while (matcher.find()) {
				String content = matcher.group(1);
				if ("scjy_rate".equals(content)) {
					if (or.getScjyRate() == null) {
						return;
					}
					key= or.getScjyRate();
				}else if ("qq_rate".equals(content)) {
					if (or.getQqRate() == null) {
						return;
					}
					key = or.getQqRate();
				}
				if (key != null) {
					replace = replace.replace("[" + content + "]", key);
				}
				placeholders.put(key, content);
			}
			String form = replace;
			CellValue cv = setFormula(st, xwb, form);
			UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
			uw.eq("id", or.getId()).set("year_rate", cv.formatAsString());
			iOrganizationRatingService.update(uw);
	   		}
		}
	}


	private CellValue setFormula(Sheet st,XSSFWorkbook xwb,String form) {
		Row rr = st.createRow(0);
		Cell cc = rr.createCell(0);
		// 这里要设置为公式类型
		cc.setCellFormula(form);
		FormulaEvaluator fe = new XSSFFormulaEvaluator(xwb);
		return fe.evaluate(cc);
	}


	/**
	 * 基建评级
	 */
	private void calcJJXMrating(List<OrganizationRatingRule> list, Map<String, List<OrganizationRating>> orgRating,Sheet st,XSSFWorkbook xwb) {
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "基建评级".equals(orr.getCheckCycle())).collect(Collectors.toList());
		for (Map.Entry<String, List<OrganizationRating>> orgMap : orgRating.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			if (collect.isEmpty()) {
				continue;
			}
			String formula=collect.get(0).getExpression();
			for (OrganizationRating or : orgMap.getValue()) {
				if (or.getCompositeScoreRaking() == null ||or.getCompositeScoreRaking().isEmpty()|| or.getProjectLevel() == null||or.getProjectLevel().isEmpty()) {
					continue;
				}
				String replace = formula;
				Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");//编译成模式
				Matcher matcher = pattern.matcher(formula);
				Map<String, String> placeholders = new HashMap<>();
				String key=null;
				while (matcher.find()) {
					String content = matcher.group(1);
					if ("ZHDEPM".equals(content)) {
						key= or.getCompositeScoreRaking();
					}else if ("JJDJ".equals(content)) {
						key = or.getProjectLevel();
					}
					if (key != null) {
						replace = replace.replace("[" + content + "]", key);
					}
					placeholders.put(key, content);
				}
				String form = replace;
				CellValue cv = setFormula(st, xwb, form);
				UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
				uw.eq("id", or.getId()).set("jj_rate", cv.formatAsString()).set("composite_score_raking",or.getCompositeScoreRaking())
					.set("project_level",or.getProjectLevel());
				iOrganizationRatingService.update(uw);
			}
		}
	}

	/**
	 * 生产经营评级
	 */
	private void calcSCJYrating(List<OrganizationRatingRule> list, Map<String, List<OrganizationRating>> orgRating,Sheet st,XSSFWorkbook xwb) {
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "生产经营评级".equals(orr.getCheckCycle())).collect(Collectors.toList());
		for (Map.Entry<String, List<OrganizationRating>> orgMap : orgRating.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			for (OrganizationRatingRule ru : rules) {
				System.out.println("key+"+orgMap.getKey()+"---"+ru.getBinSection());
			}
			if (collect.isEmpty()) {
				continue;
			}
			String formula=collect.get(0).getExpression();
		for (OrganizationRating or : orgMap.getValue()) {
			if (or.getSafetyEnvProtection() == null|| or.getSafetyEnvProtection().isEmpty() || or.getNetProfit() == null ||or.getNetProfit().isEmpty()|| or.getCompositeScore()==null ||or.getCompositeScore().isEmpty()|| or.getVeto()==null||or.getVeto().isEmpty()) {
				continue;
			}
			String replace = formula;
			Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");//编译成模式
			Matcher matcher = pattern.matcher(formula);
			Map<String, String> placeholders = new HashMap<>();
			String key=null;
			while (matcher.find()) {
				String content = matcher.group(1);
				if ("AQHB".equals(content)) {
					key= or.getSafetyEnvProtection();
				}else if ("JLR".equals(content)) {
					key = or.getNetProfit();
				}else if ("ZHDF".equals(content)) {
					key = or.getCompositeScore();
				}else if ("FJSX".equals(content)) {
					key = or.getVeto();
				}
				if (key != null) {
					replace = replace.replace("[" + content + "]", key);
				}
				placeholders.put(key, content);
			}
			String form = replace;
			CellValue cv = setFormula(st, xwb, form);
			UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
			uw.eq("id", or.getId()).set("scjy_rate", cv.formatAsString()).set("safety_env_protection",or.getSafetyEnvProtection())
				.set("veto",or.getVeto()).set("net_profit",or.getNetProfit()).set("composite_score",or.getCompositeScore());
			iOrganizationRatingService.update(uw);
		}}
	}

	/**
	 * 前期评级
	 */
	private void calcQQXMrating(List<OrganizationRatingRule> list, Map<String, List<OrganizationRating>> orgRating,Sheet st,XSSFWorkbook xwb) {
		List<OrganizationRatingRule> rules = list.stream().filter(orr -> "前期评级".equals(orr.getCheckCycle())).collect(Collectors.toList());
		for (Map.Entry<String, List<OrganizationRating>> orgMap : orgRating.entrySet()) {
			List<OrganizationRatingRule> collect = rules.stream().filter(rul -> orgMap.getKey().equals(rul.getBinSection())).collect(Collectors.toList());
			if (collect.isEmpty()) {
				continue;
			}
			String formula=collect.get(0).getExpression();
		for (OrganizationRating or : orgMap.getValue()) {
			if (or.getAnnualTarget() == null ||or.getAnnualTarget().isEmpty()|| or.getBonusPoint() == null||or.getBonusPoint().isEmpty()) {
				continue;
			}
			String replace = formula;
			Pattern pattern = Pattern.compile("\\[([^\\[\\]]+)\\]");
			Matcher matcher = pattern.matcher(formula);
			Map<String, String> placeholders = new HashMap<>();
			String key=null;
			while (matcher.find()) {
				String content = matcher.group(1);
				if ("NDMB".equals(content)) {
					 key= or.getAnnualTarget();
				}else if ("XJX".equals(content)) {
					key = or.getBonusPoint();
				}
				if (key != null) {
					replace = replace.replace("[" + content + "]", key);
				}
				placeholders.put(key, content);
			}
			String form = replace;
			CellValue cv = setFormula(st, xwb, form);
			UpdateWrapper<OrganizationRating> uw = new UpdateWrapper<>();
			uw.eq("id", or.getId()).set("qq_rate", cv.formatAsString()).set("annual_target",or.getAnnualTarget()).set("bonus_point",or.getBonusPoint());
			iOrganizationRatingService.update(uw);
		}}
	}


	@Override
	public IPage<OrganizationRating> list(Integer pageNum, Integer pageSize, String id, String
		organizationYearRatingId, String binSection, String year, String organizationShortName, String organizationId, String safetyEnvProtection,
										  String veto, String netProfit, String compositeScore, String annualTarget, String bonusPoint,String compositeScoreRaking,String projectLevel) {
		Page<OrganizationRatingRule> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		IPage<OrganizationRating> list = baseMapper.selectList(page, id, organizationYearRatingId, binSection, year, organizationShortName,
			organizationId, safetyEnvProtection, veto, netProfit, compositeScore, annualTarget, bonusPoint,compositeScoreRaking,projectLevel);
		if (list.getRecords().isEmpty()) {
			int c = iOrganizationRatingService.count();
			if (c > 0) {
				return list;
			}
			getOrgRatingInfo(organizationYearRatingId);
			list = baseMapper.selectList(page, id, organizationYearRatingId, binSection, year, organizationShortName, organizationId,
				safetyEnvProtection, veto, netProfit, compositeScore, annualTarget, bonusPoint,compositeScoreRaking,projectLevel);
		}
		return list;
	}


	/**
	 * 根据年度考评公司写年度等级表中的id
	 */
	private void getOrgRatingInfo(String organizationYearRatingId) {
		LambdaQueryWrapper<OrganizationEvaluation> lqw = new LambdaQueryWrapper<>();
		lqw.eq(OrganizationEvaluation::getCheckCycle, "NDKP");
		List<OrganizationEvaluation> ll = iOrganizationEvaluationService.list(lqw);
		if (ll.isEmpty()) {
			return;
		}
		OrganizationEvaluation yearMax = null;
		if (ll.size() > 1) {
			yearMax = ll.stream().filter(ye -> ye.getYear().matches("\\d+")).max(Comparator.comparing(ye -> Integer.parseInt(ye.getYear()))).orElse(null);
		} else {
			yearMax = ll.get(0);
		}
		if (yearMax == null) {
			return;
		}
		List<OrganizationRating> lors = new ArrayList<>();
		String ruls = yearMax.getOrganizationEvaluationRuleId();
		String[] ru = ruls.split(",");
		List<OrganizationEvaluationRule> oers = iOrganizationEvaluationRuleService.listByIds(Arrays.asList(ru));
		List<BinSection> binList = iBinSectionService.list();
		for (OrganizationEvaluationRule oer : oers) {
			if (!oer.getBinSection().contains(",")) {
				OrganizationRating or = new OrganizationRating();
				or.setYear(yearMax.getYear());
				or.setOrganizationShortName(oer.getOrganizationShortName());
				or.setOrganizationId(oer.getOrganizationId());
				or.setOrganizationYearRatingId(organizationYearRatingId);
				for (BinSection bl : binList) {
					if (bl.getId().equals(oer.getBinSection())) {
						or.setBinSection(bl.getSectionCode());
						break;
					}
				}
				lors.add(or);
				continue;
			}
			String[] split = oer.getBinSection().split(",");
			Set<String> splitSet = new LinkedHashSet<>(Arrays.asList(split));
			for (String s : splitSet) {
				OrganizationRating or = new OrganizationRating();
				or.setYear(yearMax.getYear());
				or.setOrganizationShortName(oer.getOrganizationShortName());
				or.setOrganizationId(oer.getOrganizationId());
				or.setOrganizationYearRatingId(organizationYearRatingId);
				for (BinSection bl : binList) {
					if (bl.getId().equals(s)) {
						or.setBinSection(bl.getSectionCode());
						break;
					}
				}
				lors.add(or);
			}
		}
		iOrganizationRatingService.saveBatch(lors);
	}


	@Override
	public boolean generateRating(String organizationYearRatingId, String year) {
		List<OrganizationRating> ratings = getListSating(organizationYearRatingId, year, "");
		List<String> collect = ratings.stream().map(OrganizationRating::getOrganizationId).collect(Collectors.toList());
		List<OrganizationEvaluationRule> evaluationRules = organizationEvaluationRuleService.listByIsCheck("DWKP", "NDKP", year, true, false);

		List<OrganizationRating> addRatings = new ArrayList<>();
		for (OrganizationEvaluationRule rule : evaluationRules) {
			if (null != collect && collect.size() > 0 && collect.contains(rule.getOrganizationId())) {
				continue;
			}
			OrganizationRating rating = new OrganizationRating();
			rating.setOrganizationYearRatingId(organizationYearRatingId);
			rating.setOrganizationShortName(rule.getOrganizationShortName());
			rating.setOrganizationId(rule.getOrganizationId());
			rating.setYear(year);
			rating.setYearRate("A");
			rating.setBinSection(rule.getBinSection().split(",")[0]);
			addRatings.add(rating);
		}

		if (addRatings.size() <= 0) {
			return true;
		}
		boolean b = this.saveOrUpdateBatch(addRatings);
		return b;
	}

	@Override
	public List<OrganizationRating> getListSating(String organizationYearRatingId, String year, String
		organizationId) {

		List<OrganizationRating> list = baseMapper.selectListSating(organizationYearRatingId, year, organizationId);
		return list;
	}
}
